@@ -49,18 +49,24 @@ struct occ_pstate_entry {
 	u32 freq_khz;
 };
 
+/*
+ * OCC-OPAL Shared Memory Region Version 2
+ * https://github.com/open-power/occ/blob/master/src/occ/proc/proc_pstate.h
+ * Interface defined in 'sapphire_table_t'
+ */
 struct occ_pstate_table {
 	u8 valid;
 	u8 version;
 	u8 throttle;
 	s8 pstate_min;
 	s8 pstate_nom;
-	s8 pstate_max;
-	u8 spare1;
-	u8 spare2;
+	s8 pstate_turbo;
+	s8 pstate_ultra_turbo;
+	u8 spare;
 	u64 reserved;
 	struct occ_pstate_entry pstates[MAX_PSTATES];
-};
+	s8 core_max[16];
+} __packed;
 
 DEFINE_LOG_ENTRY(OPAL_RC_OCC_LOAD, OPAL_PLATFORM_ERR_EVT, OPAL_OCC,
 		OPAL_CEC_HARDWARE, OPAL_PREDICTIVE_ERR_GENERAL,
@@ -99,6 +105,13 @@ static bool wait_for_all_occ_init(void)
 				chip->id);
 			return false;
 		}
+
+		if (!chip->occ_functional) {
+			prlog(PR_WARNING, "OCC: Chip: %x occ not functional\n",
+			      chip->id);
+			continue;
+		}
+
 		/* Get PState table address */
 		occ_data_area = chip->homer_base + P8_HOMER_SAPPHIRE_DATA_OFFSET;
 		occ_data = (struct occ_pstate_table *)occ_data_area;
@@ -135,12 +148,14 @@ static bool add_cpu_pstate_properties(s8 *pstate_nom)
 	uint64_t occ_data_area;
 	struct occ_pstate_table *occ_data;
 	struct dt_node *power_mgt;
-	u8 nr_pstates;
+	u8 nr_pstates, nr_cores = 0;
+	s8 pmax;
 	/* Arrays for device tree */
 	u32 *dt_id, *dt_freq;
 	u8 *dt_vdd, *dt_vcs;
-	bool rc;
-	int i;
+	s8 *dt_core_max = NULL;
+	bool rc, ultra_turbo_en;
+	int i, j;
 
 	prlog(PR_DEBUG, "OCC: CPU pstate state device tree init\n");
 
@@ -164,10 +179,18 @@ static bool add_cpu_pstate_properties(s8 *pstate_nom)
 		return false;
 	}
 
-	nr_pstates = occ_data->pstate_max - occ_data->pstate_min + 1;
+	if (occ_data->version > 1 &&
+	    occ_data->pstate_ultra_turbo > occ_data->pstate_turbo)
+		ultra_turbo_en = true;
+	else
+		ultra_turbo_en = false;
+
+	pmax = ultra_turbo_en ? occ_data->pstate_ultra_turbo :
+				occ_data->pstate_turbo;
+	nr_pstates = pmax - occ_data->pstate_min + 1;
 	prlog(PR_DEBUG, "OCC: Min %d Nom %d Max %d Nr States %d\n", 
 	      occ_data->pstate_min, occ_data->pstate_nom,
-	      occ_data->pstate_max, nr_pstates);
+	      pmax, nr_pstates);
 
 	if (nr_pstates <= 1 || nr_pstates > 128) {
 		prerror("OCC: OCC range is not valid\n");
@@ -184,50 +207,79 @@ static bool add_cpu_pstate_properties(s8 *pstate_nom)
 
 	/* Setup arrays for device-tree */
 	/* Allocate memory */
-	dt_id = (u32 *) malloc(MAX_PSTATES * sizeof(u32));
+	dt_id = malloc(nr_pstates * sizeof(u32));
 	if (!dt_id) {
 		printf("OCC: dt_id array alloc failure\n");
 		goto out;
 	}
 
-	dt_freq = (u32 *) malloc(MAX_PSTATES * sizeof(u32));
+	dt_freq = malloc(nr_pstates * sizeof(u32));
 	if (!dt_freq) {
 		printf("OCC: dt_freq array alloc failure\n");
 		goto out_free_id;
 	}
 
-	dt_vdd = (u8 *) malloc(MAX_PSTATES * sizeof(u8));
+	dt_vdd = malloc(nr_pstates * sizeof(u8));
 	if (!dt_vdd) {
 		printf("OCC: dt_vdd array alloc failure\n");
 		goto out_free_freq;
 	}
 
-	dt_vcs = (u8 *) malloc(MAX_PSTATES * sizeof(u8));
+	dt_vcs = malloc(nr_pstates * sizeof(u8));
 	if (!dt_vcs) {
 		printf("OCC: dt_vcs array alloc failure\n");
 		goto out_free_vdd;
 	}
 
-	for( i=0; i < nr_pstates; i++) {
-		dt_id[i] = occ_data->pstates[i].id;
-		dt_freq[i] = occ_data->pstates[i].freq_khz/1000;
-		dt_vdd[i] = occ_data->pstates[i].vdd;
-		dt_vcs[i] = occ_data->pstates[i].vcs;
+	if (ultra_turbo_en) {
+		nr_cores = get_available_nr_cores_in_chip(chip->id);
+		dt_core_max = malloc(nr_cores * sizeof(s8));
+		if (!dt_core_max) {
+			prerror("OCC: dt_core_max alloc failure\n");
+			goto out_free_vcs;
+		}
+
+		for (i = 0; i < nr_cores; i++)
+			dt_core_max[i] = occ_data->core_max[i];
+	}
+
+	for (i = 0, j = 0; i < MAX_PSTATES && j < nr_pstates; i++) {
+		if (occ_data->pstates[i].id > pmax ||
+		    occ_data->pstates[i].id < occ_data->pstate_min)
+			continue;
+		dt_id[j] = occ_data->pstates[i].id;
+		dt_freq[j] = occ_data->pstates[i].freq_khz / 1000;
+		dt_vdd[j] = occ_data->pstates[i].vdd;
+		dt_vcs[j] = occ_data->pstates[i].vcs;
+		j++;
 	}
 
 	/* Add the device-tree entries */
-	dt_add_property(power_mgt, "ibm,pstate-ids", dt_id, nr_pstates * 4);
-	dt_add_property(power_mgt, "ibm,pstate-frequencies-mhz", dt_freq, nr_pstates * 4);
+	dt_add_property(power_mgt, "ibm,pstate-ids", dt_id,
+			nr_pstates * sizeof(u32));
+	dt_add_property(power_mgt, "ibm,pstate-frequencies-mhz", dt_freq,
+			nr_pstates * sizeof(u32));
 	dt_add_property(power_mgt, "ibm,pstate-vdds", dt_vdd, nr_pstates);
 	dt_add_property(power_mgt, "ibm,pstate-vcss", dt_vcs, nr_pstates);
 	dt_add_property_cells(power_mgt, "ibm,pstate-min", occ_data->pstate_min);
 	dt_add_property_cells(power_mgt, "ibm,pstate-nominal", occ_data->pstate_nom);
-	dt_add_property_cells(power_mgt, "ibm,pstate-max", occ_data->pstate_max);
+	dt_add_property_cells(power_mgt, "ibm,pstate-max", pmax);
+
+	if (ultra_turbo_en) {
+		dt_add_property_cells(power_mgt, "ibm,pstate-turbo",
+				      occ_data->pstate_turbo);
+		dt_add_property_cells(power_mgt, "ibm,pstate-ultra-turbo",
+				      occ_data->pstate_ultra_turbo);
+		dt_add_property(power_mgt, "ibm,pstate-core-max", dt_core_max,
+				nr_cores);
+		free(dt_core_max);
+	}
 
 	/* Return pstate to set for each core */
 	*pstate_nom = occ_data->pstate_nom;
 	rc = true;
 
+out_free_vcs:
 	free(dt_vcs);
 out_free_vdd:
 	free(dt_vdd);
@@ -269,6 +321,11 @@ static bool cpu_pstates_prepare_core(struct proc_chip *chip, struct cpu_thread *
 
 	/* Set new pstate to core */
 	rc = xscom_read(chip->id, XSCOM_ADDR_P8_EX_SLAVE(core, EX_PM_PPMCR), &tmp);
+	if (rc) {
+		log_simple_error(&e_info(OPAL_RC_OCC_PSTATE_INIT),
+			"OCC: Failed to read from OCC in pstates init\n");
+		return false;
+	}
 	tmp = tmp & ~0xFFFF000000000000ULL;
 	pstate = ((uint64_t) pstate_nom) & 0xFF;
 	tmp = tmp | (pstate << 56) | (pstate << 48);
@@ -300,6 +357,12 @@ static bool cpu_pstates_prepare_core(struct proc_chip *chip, struct cpu_thread *
 
 	/* Just debug */
 	rc = xscom_read(chip->id, XSCOM_ADDR_P8_EX_SLAVE(core, EX_PM_PPMSR), &tmp);
+	if (rc) {
+		log_simple_error(&e_info(OPAL_RC_OCC_PSTATE_INIT),
+			"OCC: Failed to read back setting from OCC"
+				 "in pstates init\n");
+		return false;
+	}
 	prlog(PR_DEBUG, "OCC: Chip %x Core %x PPMSR %016llx\n",
 	      chip->id, core, tmp);
 
@@ -344,7 +407,7 @@ static void occ_throttle_poll(void *data __unused)
 			 * Queue OCC_THROTTLE with throttle status as 0 to
 			 * indicate all OCCs are active after a reset.
 			 */
-			occ_msg.type = OCC_THROTTLE;
+			occ_msg.type = cpu_to_be64(OCC_THROTTLE);
 			occ_msg.chip = 0;
 			occ_msg.throttle_status = 0;
 			rc = _opal_queue_msg(OPAL_MSG_OCC, NULL, NULL, 3,
@@ -360,9 +423,9 @@ static void occ_throttle_poll(void *data __unused)
 			if ((occ_data->valid == 1) &&
 			    (chip->throttle != occ_data->throttle) &&
 			    (occ_data->throttle <= OCC_MAX_THROTTLE_STATUS)) {
-				occ_msg.type = OCC_THROTTLE;
-				occ_msg.chip = chip->id;
-				occ_msg.throttle_status = occ_data->throttle;
+				occ_msg.type = cpu_to_be64(OCC_THROTTLE);
+				occ_msg.chip = cpu_to_be64(chip->id);
+				occ_msg.throttle_status = cpu_to_be64(occ_data->throttle);
 				rc = _opal_queue_msg(OPAL_MSG_OCC, NULL,
 						     occ_msg_consumed,
 						     3, (uint64_t *)&occ_msg);
@@ -436,6 +499,50 @@ struct occ_load_req {
 };
 static LIST_HEAD(occ_load_req_list);
 
+int find_master_and_slave_occ(uint64_t **master, uint64_t **slave,
+			      int *nr_masters, int *nr_slaves)
+{
+	struct proc_chip *chip;
+	int nr_chips = 0, i;
+	uint64_t chipids[MAX_CHIPS];
+
+	for_each_chip(chip) {
+		chipids[nr_chips++] = chip->id;
+	}
+
+	chip = next_chip(NULL);
+	/*
+	 * Proc0 is the master OCC for Tuleta/Alpine boxes.
+	 * Hostboot expects the pair of chips for MURANO, so pass the sibling
+	 * chip id along with proc0 to hostboot.
+	 */
+	*nr_masters = (chip->type == PROC_CHIP_P8_MURANO) ? 2 : 1;
+	*master = (uint64_t *)malloc(*nr_masters * sizeof(uint64_t));
+
+	if (!*master) {
+		printf("OCC: master array alloc failure\n");
+		return -ENOMEM;
+	}
+
+	if (nr_chips - *nr_masters > 0) {
+		*nr_slaves = nr_chips - *nr_masters;
+		*slave = (uint64_t *)malloc(*nr_slaves * sizeof(uint64_t));
+		if (!*slave) {
+			printf("OCC: slave array alloc failure\n");
+			return -ENOMEM;
+		}
+	}
+
+	for (i = 0; i < nr_chips; i++) {
+		if (i < *nr_masters) {
+			*(*master + i) = chipids[i];
+			continue;
+		}
+		*(*slave + i - *nr_masters) = chipids[i];
+	}
+	return 0;
+}
+
 static void occ_queue_load(u8 scope, u32 dbob_id, u32 seq_id)
 {
 	struct occ_load_req *occ_req;
@@ -467,7 +574,7 @@ static void __occ_do_load(u8 scope, u32 dbob_id __unused, u32 seq_id)
 		prlog(PR_INFO, "OCC: Load: Fallback to preloaded image\n");
 		rc = 0;
 	} else if (!rc) {
-		struct opal_occ_msg occ_msg = { OCC_LOAD, 0, 0 };
+		struct opal_occ_msg occ_msg = { CPU_TO_BE64(OCC_LOAD), 0, 0 };
 
 		rc = _opal_queue_msg(OPAL_MSG_OCC, NULL, NULL, 3,
 				     (uint64_t *)&occ_msg);
